"""
 A form action adapter that e-mails input.
"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'


########################
# The formMailerAdapter code and schema borrow heavily
# from PloneFormMailer <http://plone.org/products/ploneformmailer>
# by Jens Klein and Reinout van Rees.
#
# Author:       Jens Klein <jens.klein@jensquadrat.com>
#
# Copyright:    (c) 2004 by jens quadrat, Klein & Partner KEG, Austria
# Licence:      GNU General Public Licence (GPL) Version 2 or later
#######################

from AccessControl import ClassSecurityInfo

from Products.Archetypes.public import *
from Products.Archetypes.utils import OrderedDict
from Products.Archetypes.utils import shasattr

from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.base import registerATCT

from Products.CMFCore.permissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

from Products.PloneFormGen.config import *
from Products.PloneFormGen.content.actionAdapter import FormActionAdapter, FormAdapterSchema

from Products.TALESField import TALESString
from Products.TemplateFields import ZPTField as ZPTField

from ya_gpg import gpg

from email import Encoders
from email.Header import Header
from email.MIMEAudio import MIMEAudio
from email.MIMEBase import MIMEBase
from email.MIMEImage import MIMEImage
from email.MIMEMultipart import MIMEMultipart
from email.MIMEText import MIMEText

from types import StringTypes

from Products.PloneFormGen import PloneFormGenMessageFactory as _
from Products.PloneFormGen import HAS_PLONE30, dollarReplace

try:
    # 3.0+
    from zope.contenttype import guess_content_type
except ImportError:
    # 2.5.x
    from zope.app.content_types import guess_content_type

import zope.i18n


formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
    StringField('recipient_name',
        searchable=0,
        required=0,
        default_method='getDefaultRecipientName',
        write_permission=EDIT_ADDRESSING_PERMISSION,
        read_permission=ModifyPortalContent,
        widget=StringWidget(
            label = "Recipient's full name",
            description = """
                The full name of the recipient of the mailed form.
                """,
            description_msgid = "help_formmailer_recipient_fullname",
            label_msgid = "label_formmailer_recipient_fullname",
            i18n_domain = "ploneformgen",
            ),
        ),
    StringField('recipient_email',
        searchable=0,
        required=0,
        default_method='getDefaultRecipient',
        write_permission=EDIT_ADDRESSING_PERMISSION,
        read_permission=ModifyPortalContent,
        validators=('isEmail',),
        widget=StringWidget(
            label = "Recipient's e-mail address",
            description = 'The recipients e-mail address.',
            description_msgid = "help_formmailer_recipient_email",
            label_msgid = "label_formmailer_recipient_email",
            i18n_domain = "ploneformgen",
            ),
        ),
    StringField('to_field',
        schemata='addressing',
        searchable=0,
        required=0,
        default='#NONE#',
        write_permission=EDIT_ADVANCED_PERMISSION,
        read_permission=ModifyPortalContent,
        vocabulary='fieldsDisplayList',
        widget=SelectionWidget(
            label = 'Extract Recipient From',
            label_msgid = "label_formmailer_to_extract",
            description =
                """
                Choose a form field from which you wish to extract
                input for the To header. If you choose anything other
                than "None", this will override the "Recipient's e-mail address"
                setting above. Be very cautious about allowing unguarded user
                input for this purpose.
                """,
            description_msgid = "help_formmailer_to_extract",
            i18n_domain = "ploneformgen",
            ),
        ),
    LinesField('cc_recipients',
        searchable=0,
        required=0,
        default_method='getDefaultCC',
        schemata='addressing',
        write_permission=EDIT_ADDRESSING_PERMISSION,
        read_permission=ModifyPortalContent,
        widget=LinesWidget(
            label = 'CC Recipients',
            description = 'E-mail addresses which receive a carbon copy.',
            description_msgid = "help_formmailer_cc_recipients",
            label_msgid = "label_formmailer_cc_recipients",
            i18n_domain = "ploneformgen",
            ),
        ),
    LinesField('bcc_recipients',
        schemata='addressing',
        searchable=0,
        required=0,
        default_method='getDefaultBCC',
        write_permission=EDIT_ADDRESSING_PERMISSION,
        read_permission=ModifyPortalContent,
        widget=LinesWidget(
            label = 'BCC Recipients',
            description = 'E-mail addresses which receive a blind carbon copy.',
            description_msgid = "help_formmailer_bcc_recipients",
            label_msgid = "label_formmailer_bcc_recipients",
            i18n_domain = "ploneformgen",
            ),
        ),
    StringField('replyto_field',
        schemata='addressing',
        searchable=0,
        required=0,
        vocabulary='fieldsDisplayList',
        read_permission=ModifyPortalContent,
        write_permission=EDIT_ADVANCED_PERMISSION,
        widget=SelectionWidget(
            label = 'Extract Reply-To From',
            label_msgid = "label_formmailer_replyto_extract",
            description =
                """
                Choose a form field from which you wish to extract
                input for the Reply-To header. NOTE: You should
                activate e-mail address verification for the designated
                field.
                """,
            description_msgid = "help_formmailer_replyto_extract",
            i18n_domain = "ploneformgen",
            ),
        ),
    StringField('msg_subject',
        schemata='message',
        searchable=0,
        required=0,
        default='Form Submission',
        read_permission=ModifyPortalContent,
        widget=StringWidget(
            description =
                """
                Subject line of message. This is used if you
                do not specify a subject field or if the field
                is empty.
                """,
            description_msgid = "help_formmailer_subject",
            label = 'Subject',
            label_msgid = "label_formmailer_subject",
            i18n_domain = "ploneformgen",
            ),
        ),
    StringField('subject_field',
        schemata='message',
        searchable=0,
        required=0,
        vocabulary='fieldsDisplayList',
        write_permission=EDIT_ADVANCED_PERMISSION,
        read_permission=ModifyPortalContent,
        widget=SelectionWidget(
            label = 'Extract Subject From',
            label_msgid = "label_formmailer_subject_extract",
            description =
                """
                Choose a form field from which you wish to extract
                input for the mail subject line.
                """,
            description_msgid = "help_formmailer_subject_extract",
            i18n_domain = "ploneformgen",
            ),
        ),
    TextField('body_pre',
        searchable=0,
        required=0,
        schemata='message',
        accessor='getBody_pre',
        read_permission=ModifyPortalContent,
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        widget=TextAreaWidget(description = 'Text prepended to fields listed in mail-body',
            description_msgid = "help_formmailer_body_pre",
            label = 'Body (prepended)',
            label_msgid = "label_formmailer_body_pre",
            i18n_domain = "ploneformgen",
            ),
        ),
    TextField('body_post',
        searchable=0,
        required=0,
        schemata='message',
        read_permission=ModifyPortalContent,
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        widget=TextAreaWidget(description = 'Text appended to fields listed in mail-body',
            description_msgid = "help_formmailer_body_post",
            label = 'Body (appended)',
            label_msgid = "label_formmailer_body_post",
            i18n_domain = "ploneformgen",
            ),
        ),

    TextField('body_footer',
        searchable=0,
        required=0,
        schemata='message',
        read_permission=ModifyPortalContent,
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        widget=TextAreaWidget(description = 'Text used as the footer at '
            'bottom, delimited from the body by a dashed line.',
            description_msgid = "help_formmailer_body_footer",
            label = 'Body (signature)',
            label_msgid = "label_formmailer_body_footer",
            i18n_domain = "ploneformgen",
            ),
        ),
    BooleanField('showAll',
        required=0,
        searchable=0,
        schemata='message',
        default='1',
        widget=BooleanWidget(
            label="Include All Fields",
            description="""
                Check this to include input for all fields
                (except label and file fields). If you check
                this, the choices in the pick box below
                will be ignored.
                """,
            label_msgid = "label_mailallfields_text",
            description_msgid = "help_mailallfields_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    LinesField('showFields',
        required=0,
        searchable=0,
        schemata='message',
        vocabulary='allFieldDisplayList',
        widget=PicklistWidget(
            label="Show Responses",
            description="""
                Pick the fields whose inputs you'd like to include in
                the e-mail.
                """,
            label_msgid = "label_mailfields_text",
            description_msgid = "help_mailfields_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    BooleanField('includeEmpties',
        required=0,
        searchable=0,
        schemata='message',
        default='1',
        widget=BooleanWidget(
            label="Include Empties",
            description="""
                Check this to include titles
                for fields that received no input. Uncheck
                to leave fields with no input out of the e-mail.
                """,
            label_msgid = "label_mailEmpties_text",
            description_msgid = "help_mailEmpties_text",
            i18n_domain = "ploneformgen",
            ),
        ),
    ZPTField('body_pt',
        schemata='template',
        write_permission=EDIT_TALES_PERMISSION,
        default_method = 'getMailBodyDefault',
        read_permission=ModifyPortalContent,
        widget=TextAreaWidget(description = 'This is a Zope Page Template '
            'used for rendering of the mail-body. You don\'t need to modify '
            'it, but if you know TAL (Zope\'s Template Attribute Language) '
            'you have the full power to customize your outgoing mails.',
            description_msgid = "help_formmailer_body_pt",
            label = 'Mail-Body Template',
            label_msgid = "label_formmailer_body_pt",
            i18n_domain = "ploneformgen",
            rows = 20,
            visible = {'edit':'visible','view':'invisible'},
            ) ,
        validators=('zptvalidator',),
        ),
    StringField('body_type',
        schemata='template',
        default_method='getMailBodyTypeDefault',
        vocabulary = MIME_LIST,
        write_permission=EDIT_ADVANCED_PERMISSION,
        read_permission=ModifyPortalContent,
        widget=SelectionWidget(description = 'Set the mime-type of the mail-body. '
            'Change this setting only if you know exactly what you are doing. '
            'Leave it blank for default behaviour.',
            description_msgid = "help_formmailer_body_type",
            label = 'Mail-Body Type',
            label_msgid = "label_formmailer_body_type",
            i18n_domain = "ploneformgen",
            ),
        ),
    LinesField('xinfo_headers',
        searchable=0,
        required=0,
        schemata='headers',
        default_method='getDefaultXInfo',
        write_permission=EDIT_ADVANCED_PERMISSION,
        read_permission=ModifyPortalContent,
        vocabulary=DisplayList( (
            ('HTTP_X_FORWARDED_FOR','HTTP_X_FORWARDED_FOR',),
            ('REMOTE_ADDR','REMOTE_ADDR',),
            ('PATH_INFO', 'PATH_INFO'),
            ('HTTP_USER_AGENT','HTTP_USER_AGENT',),
            ('HTTP_REFERER', 'HTTP_REFERER'),
            ), ),
        widget=MultiSelectionWidget(
            label='HTTP Headers',
            description="""
                Pick any items from the HTTP headers that
                you'd like to insert as X- headers in the
                message.
                """,
            format='checkbox',
            i18n_domain = "ploneformgen",
            label_msgid = "label_xinfo_headers_text",
            description_msgid = "help_xinfo_headers_text",
            ),
        ),
    LinesField('additional_headers',
        schemata='headers',
        searchable=0,
        required=0,
        default_method='getDefaultAddHdrs',
        write_permission=EDIT_ADVANCED_PERMISSION,
        read_permission=ModifyPortalContent,
        widget=LinesWidget(
            label = 'Additional Headers',
            description="""
                Additional e-mail-header lines.
                Only use RFC822-compliant headers.
                """,
            description_msgid = "help_formmailer_additional_headers",
            label_msgid = "label_formmailer_additional_headers",
            i18n_domain = "ploneformgen",
            ),
        ),
))

if gpg is not None:
    formMailerAdapterSchema = formMailerAdapterSchema + Schema((
        StringField('gpg_keyid',
            schemata='encryption',
            accessor='getGPGKeyId',
            mutator='setGPGKeyId',
            write_permission=USE_ENCRYPTION_PERMISSION,
            read_permission=ModifyPortalContent,
            widget=StringWidget(
                description = """
                    Give your key-id, e-mail address or
                    whatever works to match a public key from current keyring.
                    It will be used to encrypt the message body.
                    Contact the site administrator if you need to
                    install a new public key.
                    Note that you will probably wish to change your message
                    template to plain text if you're using encryption.
                    TEST THIS FEATURE BEFORE GOING PUBLIC!
                    """,
                description_msgid = "help_gpg_key_id",
                label = 'Key-Id',
                label_msgid = "label_gpg_key_id",
                i18n_domain='ploneformgen',
                ),
            ),
        ))


formMailerAdapterSchema = formMailerAdapterSchema + Schema((
    TALESString('subjectOverride',
        schemata='overrides',
        searchable=0,
        required=0,
        validators=('talesvalidator',),
        default='',
        write_permission=EDIT_TALES_PERMISSION,
        read_permission=ModifyPortalContent,
        isMetadata=True, # just to hide from base view
        widget=StringWidget(label="Subject Expression",
            description="""
                A TALES expression that will be evaluated to override any value
                otherwise entered for the e-mail subject header.
                Leave empty if unneeded. Your expression should evaluate as a string.
                PLEASE NOTE: errors in the evaluation of this expression will cause
                an error on form display.
            """,
            size=70,
            i18n_domain = "ploneformgen",
            label_msgid = "label_subject_override_text",
            description_msgid = "help_subject_override_text",
        ),
    ),
    TALESString('senderOverride',
        schemata='overrides',
        searchable=0,
        required=0,
        validators=('talesvalidator',),
        default='',
        write_permission=EDIT_TALES_PERMISSION,
        read_permission=ModifyPortalContent,
        isMetadata=True, # just to hide from base view
        widget=StringWidget(label="Sender Expression",
            description="""
                A TALES expression that will be evaluated to override the "From" header.
                Leave empty if unneeded. Your expression should evaluate as a string.
                PLEASE NOTE: errors in the evaluation of this expression will cause
                an error on form display.
            """,
            size=70,
            i18n_domain = "ploneformgen",
            label_msgid = "label_sender_override_text",
            description_msgid = "help_sender_override_text",
        ),
    ),
    TALESString('recipientOverride',
        schemata='overrides',
        searchable=0,
        required=0,
        validators=('talesvalidator',),
        default='',
        write_permission=EDIT_TALES_PERMISSION,
        read_permission=ModifyPortalContent,
        isMetadata=True, # just to hide from base view
        widget=StringWidget(label="Recipient Expression",
            description="""
                A TALES expression that will be evaluated to override any value
                otherwise entered for the recipient e-mail address. You are strongly
                cautioned against using unvalidated data from the request for this purpose.
                Leave empty if unneeded. Your expression should evaluate as a string.
                PLEASE NOTE: errors in the evaluation of this expression will cause
                an error on form display.
            """,
            size=70,
            i18n_domain = "ploneformgen",
            label_msgid = "label_recipient_override_text",
            description_msgid = "help_recipient_override_text",
        ),
    ),
    TALESString('bccOverride',
        schemata='overrides',
        searchable=0,
        required=0,
        validators=('talesvalidator',),
        default='',
        write_permission=EDIT_TALES_PERMISSION,
        read_permission=ModifyPortalContent,
        isMetadata=True, # just to hide from base view
        widget=StringWidget(label="BCC Expression",
            description="""
                A TALES expression that will be evaluated to override any value
                otherwise entered for the BCC list. You are strongly
                cautioned against using unvalidated data from the request for this purpose.
                Leave empty if unneeded. Your expression should evaluate as a sequence of string.
                PLEASE NOTE: errors in the evaluation of this expression will cause
                an error on form display.
            """,
            size=70,
            i18n_domain = "ploneformgen",
            label_msgid = "label_bcc_override_text",
            description_msgid = "help_bcc_override_text",
        ),
    ),
))

if HAS_PLONE30:
    # move headers schema items to template schema to keep the schema
    # count <= 6
    formMailerAdapterSchema['xinfo_headers'].schemata = 'template'
    formMailerAdapterSchema['additional_headers'].schemata = 'template'

formMailerAdapterSchema.moveField('execCondition', pos='bottom')
if not HAS_PLONE30:
    finalizeATCTSchema(formMailerAdapterSchema, folderish=True, moveDiscussion=False)


class FormMailerAdapter(FormActionAdapter):
    """ A form action adapter that will e-mail form input. """

    schema = formMailerAdapterSchema
    portal_type = meta_type = 'FormMailerAdapter'
    archetype_name = 'Mailer Adapter'
    content_icon = 'mailaction.gif'

    security       = ClassSecurityInfo()


    def initializeArchetype(self, **kwargs):
        """ Translate the adapter in the current langage
        """

        FormActionAdapter.initializeArchetype(self, **kwargs)

        self.setMsg_subject(zope.i18n.translate(_(u'pfg_formmaileradapter_msg_subject', u'Form Submission'), context=self.REQUEST))


    security.declarePrivate('onSuccess')
    def onSuccess(self, fields, REQUEST=None):
        """
        e-mails data.
        """

        self.send_form(fields, REQUEST)


    security.declarePrivate('getMailBodyDefault')
    def getMailBodyDefault(self):
        """ Get default mail body from our tool """

        fgt = getToolByName(self, 'formgen_tool')
        return fgt.getDefaultMailTemplateBody()


    security.declarePrivate('getMailBodyTypeDefault')
    def getMailBodyTypeDefault(self):
        """ Get default mail body type from our tool """

        fgt = getToolByName(self, 'formgen_tool')
        return fgt.getDefaultMailBodyType()


    security.declarePrivate('getDefaultRecipient')
    def getDefaultRecipient(self):
        """ Get default mail recipient from our tool """

        fgt = getToolByName(self, 'formgen_tool')
        return fgt.getDefaultMailRecipient()


    security.declarePrivate('getDefaultRecipientName')
    def getDefaultRecipientName(self):
        """ Get default mail recipient from our tool """

        fgt = getToolByName(self, 'formgen_tool')
        return fgt.getDefaultMailRecipientName()


    security.declarePrivate('getDefaultCC')
    def getDefaultCC(self):
        """ Get default mail cc from our tool """

        fgt = getToolByName(self, 'formgen_tool')
        return fgt.getDefaultMailCC()


    security.declarePrivate('getDefaultBCC')
    def getDefaultBCC(self):
        """ Get default mail bcc from our tool """

        fgt = getToolByName(self, 'formgen_tool')
        return fgt.getDefaultMailBCC()


    security.declarePrivate('getDefaultXInfo')
    def getDefaultXInfo(self):
        """ Get default mail xinfo hdrs from our tool """

        fgt = getToolByName(self, 'formgen_tool')
        return fgt.getDefaultMailXInfo()


    security.declarePrivate('getDefaultAddHdrs')
    def getDefaultAddHdrs(self):
        """ Get default mail add headers from our tool """

        fgt = getToolByName(self, 'formgen_tool')
        return fgt.getDefaultMailAddHdrs()


    security.declarePublic('setBody_pt')
    def setBody_pt(self, value, **kw):
        """ set body template with BBB for accessors """
        
        if shasattr(value, 'replace'):
            template = value
            for s, t in [('body_pre', 'getBody_pre'),
                         ('body_post', 'getBody_post'), 
                         ('body_footer', 'getBody_footer')]:
                template = template.replace('here/%s' % s, 'here/%s' % t)
            myField = self.getField('body_pt')
            myField.set(self, template)


    security.declarePrivate('_dreplace')
    def _dreplace(self, s):
        return dollarReplace.DollarVarReplacer(getattr(self.REQUEST, 'form', {})).sub(s)


    security.declarePublic('getBody_pre')
    def getBody_pre(self):
        """ get expanded mail body prefix """
        
        return self._dreplace( self.getRawBody_pre() )


    security.declarePublic('getBody_post')
    def getBody_post(self):
        """ get expanded mail body postfix """

        return self._dreplace( self.getRawBody_post() )


    security.declarePublic('getBody_footer')
    def getBody_footer(self):
        """ get expanded mail body footer """

        return self._dreplace( self.getRawBody_footer() )


    security.declarePrivate('get_mail_text')
    def get_mail_text(self, fields, request, **kwargs):
        """Get header and body of e-mail as text (string)
        """
        
        (headerinfo, additional_headers,
         body) = self.get_header_body_tuple(fields, request, **kwargs)

        if not isinstance(body, unicode):
            body = unicode(body, self._site_encoding())
        portal = getToolByName(self, 'portal_url').getPortalObject()
        email_charset = portal.getProperty('email_charset', 'utf-8')
        mime_text = MIMEText(body.encode(email_charset , 'replace'),
                _subtype=self.body_type or 'html', _charset=email_charset)

        attachments = self.get_attachments(fields, request)

        if attachments:
            outer = MIMEMultipart()
            outer.attach(mime_text)
        else:
            outer = mime_text

        # write header
        for key, value in headerinfo.items():
            outer[key] = value

        # write additional header
        for a in additional_headers:
            key, value = a.split(':', 1)
            outer.add_header(key, value.strip())

        for attachment in attachments:
            filename = attachment[0]
            ctype = attachment[1]
            encoding = attachment[2]
            content = attachment[3]

            if ctype is None:
                ctype = 'application/octet-stream'

            maintype, subtype = ctype.split('/', 1)

            if maintype == 'text':
                msg = MIMEText(content, _subtype=subtype)
            elif maintype == 'image':
                msg = MIMEImage(content, _subtype=subtype)
            elif maintype == 'audio':
                msg = MIMEAudio(content, _subtype=subtype)
            else:
                msg = MIMEBase(maintype, subtype)
                msg.set_payload(content)
                # Encode the payload using Base64
                Encoders.encode_base64(msg)

            # Set the filename parameter
            msg.add_header('Content-Disposition', 'attachment', filename=filename)
            outer.attach(msg)

        return outer.as_string()


    def get_attachments(self, fields, request):
        """Return all attachments uploaded in form.
        """

        from ZPublisher.HTTPRequest import FileUpload

        attachments = []

        for field in fields:
            if field.isFileField():
                file = request.form.get('%s_file' % field.__name__, None)
                if file and isinstance(file, FileUpload) and file.filename != '':
                    file.seek(0) # rewind
                    data = file.read()
                    filename = file.filename
                    mimetype, enc = guess_content_type(filename, data, None)
                    attachments.append((filename, mimetype, enc, data))
        return attachments

    security.declarePrivate('get_mail_body')
    def get_mail_body(self, fields, **kwargs):
        """Returns the mail-body with footer.
        """

        if kwargs.has_key('request'):
            request = kwargs['request']
        else:
            request = self.REQUEST

        all_fields = [f for f in fields
            if not (f.isLabel() or f.isFileField()) and not (getattr(self, 'showAll', True) and f.getServerSide())]

        # which fields should we show?
        if getattr(self, 'showAll', True):
            live_fields = all_fields 
        else:
            live_fields = \
                [f for f in all_fields
                   if f.fgField.getName() in getattr(self, 'showFields', ())]

        if not getattr(self, 'includeEmpties', True):
            all_fields = live_fields
            live_fields = []
            for f in all_fields:
                value = f.htmlValue(request)
                if value and value != 'No Input':
                    live_fields.append(f)
                
        bare_fields = [f.fgField for f in live_fields]
        bodyfield = self.getField('body_pt')
        
        # pass both the bare_fields (fgFields only) and full fields.
        # bare_fields for compatability with older templates,
        # full fields to enable access to htmlValue
        body = bodyfield.get(self, fields=bare_fields, wrappedFields=live_fields,**kwargs)

        if isinstance(body, unicode):
            body = body.encode(self.getCharset())

        keyid = getattr(self, 'gpg_keyid', None)
        encryption = gpg and keyid

        if encryption:
            bodygpg = gpg.encrypt(body, keyid)
            if bodygpg.strip():
                body = bodygpg

        return body


    security.declarePrivate('secure_header_line')
    def secure_header_line(self, line):
        nlpos = line.find('\x0a')
        if nlpos >= 0:
            line = line[:nlpos]
        nlpos = line.find('\x0d')
        if nlpos >= 0:
            line = line[:nlpos]
        return line


    security.declarePrivate('_destFormat')
    def _destFormat(self, input):
        """ Format destination (To) input.
            Input may be a string or sequence of strings;
            returns a well-formatted address field
        """

        if type(input) in StringTypes:
            input = [s for s in input.split(',')]
        input = [s for s in input if s]
        filtered_input = [s.strip().encode('utf-8') for s in input]
        
        if filtered_input:        
            return "<%s>" % '>, <'.join( filtered_input )
        else:
            return ''


    security.declarePrivate('get_header_body_tuple')
    def get_header_body_tuple(self, fields, request,
                              from_addr=None, to_addr=None,
                              subject=None, **kwargs):
        """Return header and body of e-mail as an 3-tuple:
        (header, additional_header, body)

        header is a dictionary, additional header is a list, body is a StringIO

        Keyword arguments:
        request -- (optional) alternate request object to use
        """

        pprops = getToolByName(self, 'portal_properties')
        site_props = getToolByName(pprops, 'site_properties')
        portal = getToolByName(self, 'portal_url').getPortalObject()
        pms = getToolByName(self, 'portal_membership')
        utils = getToolByName(self, 'plone_utils')
        
        body = self.get_mail_body(fields, **kwargs)

        # fields = self.fgFields()

        # get Reply-To
        reply_addr = None
        if shasattr(self, 'replyto_field'):
            reply_addr = request.form.get(self.replyto_field, None)

        # get subject header
        nosubject = '(no subject)'
        if shasattr(self, 'subjectOverride') and self.getRawSubjectOverride():
            # subject has a TALES override
            subject = self.getSubjectOverride().strip()
        else:
            subject = getattr(self, 'msg_subject', nosubject)
            subjectField = request.form.get(self.subject_field, None)
            if subjectField is not None:
                subject = subjectField
            else:
                # we only do subject expansion if there's no field chosen
                subject = self._dreplace(subject)

        # Get From address
        if shasattr(self, 'senderOverride') and self.getRawSenderOverride():
            from_addr = self.getSenderOverride().strip()
        else:
            from_addr = from_addr or site_props.getProperty('email_from_address') or \
                        portal.getProperty('email_from_address')

        # Get To address and full name
        if shasattr(self, 'recipientOverride') and self.getRawRecipientOverride():
            recip_email = self.getRecipientOverride()
        else:
            recip_email = None
            if shasattr(self, 'to_field'):
                recip_email = request.form.get(self.to_field, None)
            if not recip_email:
                recip_email = self.recipient_email
        recip_email = self._destFormat( recip_email )

        recip_name = self.recipient_name.encode('utf-8')

        # if no to_addr and no recip_email specified, use owner adress if possible.
        # if not, fall back to portal email_from_address.
        # if still no destination, raise an assertion exception.
        if not recip_email and not to_addr:
            ownerinfo = self.getOwner()
            ownerid=ownerinfo.getId()
            userdest = pms.getMemberById(ownerid)
            toemail = userdest.getProperty('email', '')
            if not toemail:
                toemail = portal.getProperty('email_from_address')                
            assert toemail, """
                    Unable to mail form input because no recipient address has been specified.
                    Please check the recipient settings of the PloneFormGen "Mailer" within the
                    current form folder.
                """
            to = '%s <%s>' %(ownerid,toemail)
        else:
            to = to_addr or '%s %s' % (recip_name, recip_email)

        headerinfo = OrderedDict()

        headerinfo['To'] = self.secure_header_line(to)
        headerinfo['From'] = self.secure_header_line(from_addr)
        if reply_addr:
            headerinfo['Reply-To'] = self.secure_header_line(reply_addr)

        # transform subject into mail header encoded string
        email_charset = portal.getProperty('email_charset', 'utf-8')

        if not isinstance(subject, unicode):
            site_charset = utils.getSiteEncoding()            
            subject = unicode(subject, site_charset, 'replace')        
        
        msgSubject = self.secure_header_line(subject).encode(email_charset, 'replace')
        msgSubject = str(Header(msgSubject, email_charset))
        headerinfo['Subject'] = msgSubject

        headerinfo['MIME-Version'] = '1.0'

        # CC
        cc_recips = filter(None, self.cc_recipients)
        if cc_recips:
            headerinfo['Cc'] = self._destFormat( cc_recips )

        # BCC
        bcc_recips = filter(None, self.bcc_recipients)
        if shasattr(self, 'bccOverride') and self.getRawBccOverride():
            bcc_recips = self.getBccOverride()
        if bcc_recips:
            headerinfo['Bcc'] = self._destFormat( bcc_recips )

        for key in getattr(self, 'xinfo_headers', []):
            headerinfo['X-%s' % key] = self.secure_header_line(request.get(key, 'MISSING'))

        # return 3-Tuple
        return (headerinfo, self.additional_headers, body)


    def send_form(self, fields, request, **kwargs):
        """Send the form.
        """

        mailtext=self.get_mail_text(fields, request, **kwargs)
        host = self.MailHost
        host.send(mailtext)


    # translation and encodings
    def _site_encoding(self):
        site_props = self.portal_properties.site_properties
        return site_props.default_charset or 'UTF-8'


    security.declareProtected(View, 'allFieldDisplayList')
    def allFieldDisplayList(self):
        """ returns a DisplayList of all fields """

        return self.fgFieldsDisplayList()


    def fieldsDisplayList(self):
        """ returns display list of fields with simple values """

        return self.fgFieldsDisplayList(
            withNone=True,
            noneValue='#NONE#',
            objTypes=(
                'FormSelectionField',
                'FormStringField',
                )
            )


    security.declareProtected(ModifyPortalContent, 'setShowFields')
    def setShowFields(self, value, **kw):
        """ Reorder form input to match field order """
        # This wouldn't be desirable if the PickWidget
        # retained order.

        self.showFields = []
        for field in self.fgFields(excludeServerSide=False):
            id = field.getName()
            if id in value:
                self.showFields.append(id)


registerATCT(FormMailerAdapter, PROJECTNAME)



from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget
from AccessControl import ClassSecurityInfo


class CaptchaWidget(TypesWidget):
    """ Archetypes widget wrapping collective.captcha or collective.recaptcha
    """

    _properties = TypesWidget._properties.copy()
    _properties.update({
        'macro' : 'widget_captcha',
        'size' : '30',
        'maxlength' : '255',
        'blurrable' : True,
        'postback' : False,
        })

    security = ClassSecurityInfo()

# Register the widget with Archetypes
registerWidget(CaptchaWidget,
               title = 'Captcha widget',
               description= ('Renders a captcha image and a text input box.',),
               used_for = ('Products.Archetypes.Field.StringField',)
               )



""" Base declarations and classes for form fields """

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

import cgi

from types import StringTypes, BooleanType

import transaction
import zExceptions

from zope.interface import implements

from Products.Archetypes.public import *
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.configuration import zconf

from Products.TALESField import TALESString, TALESLines

from Products.PloneFormGen.config import \
    PROJECTNAME, EDIT_TALES_PERMISSION, EDIT_ADVANCED_PERMISSION

from Products.CMFCore.permissions import View, ModifyPortalContent
from Products.CMFCore.Expression import getExprContext
from Products.CMFCore.exceptions import BadRequest
from Products.CMFCore.utils import getToolByName

from Products.CMFPlone.utils import base_hasattr, safe_hasattr

from Products.validation.validators import RangeValidator
from AccessControl import ClassSecurityInfo

from Products.PloneFormGen.content import validationMessages
from Products.PloneFormGen.interfaces import IPloneFormGenField

from Products.PloneFormGen import PloneFormGenMessageFactory as _
from Products.PloneFormGen import HAS_PLONE30


def finalizeFieldSchema(schema, folderish=True, moveDiscussion=False):
    """ cleanup typical field schema """

    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
    if HAS_PLONE30:
        # avoid showing unnecessary schema tabs
        for afield in ('subject', 
                       'relatedItems', 
                       'location', 
                       'language', 
                       'effectiveDate', 
                       'expirationDate', 
                       'creation_date', 
                       'modification_date', 
                       'creators', 
                       'contributors', 
                       'rights', 
                       'allowDiscussion', 
                       'excludeFromNav', ):
            schema[afield].widget.visible = {'view':'invisible','edit':'invisible'}
            schema[afield].schemata = 'default'


###
# we use these fields in several schemata

validatorOverrideField = \
        TALESString('fgTValidator',
            schemata='overrides',
            searchable=0,
            required=0,
            validators=('talesvalidator',),
            default="python:False",
            write_permission=EDIT_TALES_PERMISSION,
            widget=StringWidget(label="Custom Validator",
                description="""
                    A TALES expression that will be evaluated when the form is validated.
                    Validate against 'value', which will contain the field input.
                    Return False if valid; if not valid return a string error message.
                    E.G., "python: test(value=='eggs', False, 'input must be eggs')" will
                    require "eggs" for input.
                    PLEASE NOTE: errors in the evaluation of this expression will cause
                    an error on form display.
                """,
                size=70,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgtvalidator_text",
                description_msgid = "help_fgtvalidator_text",
                ),
            )

rowsField = \
    IntegerField('fgRows',
        searchable=0,
        required=0,
        default='5',
        widget=IntegerWidget(label='Rows',
            i18n_domain = "ploneformgen",
            label_msgid = "label_rows_text",
            description_msgid = "help_rows_text",
            ),
        )

maxlengthField = \
    IntegerField('fgmaxlength',
        default=255,
        widget=IntegerWidget(
            label='Max Length',
            description="""
                The maximum number of characters the user will be able to input.
                """,
            i18n_domain = "ploneformgen",
            label_msgid = "label_fgmaxlength_text",
            description_msgid = "help_fgmaxlength_text",
            ),
        )

maxlengthField0 = \
        IntegerField('fgmaxlength',
            default=0,
            widget=IntegerWidget(
                label='Max Length',
                description="""
                    The maximum number of characters the user will be able to input.
                    Use 0 for no limit.
                    """,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgmaxlength_text",
                description_msgid = "help_fgmaxlength_text",
                ),
            )

maxlengthField4k = \
        IntegerField('fgmaxlength',
            default='4096',
            widget=IntegerWidget(
                label='Max Length',
                description="""
                    The maximum number of characters the user will be able to input.
                    Use 0 for no limit.
                    """,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgmaxlength_text",
                description_msgid = "help_fgmaxlength_text",
                ),
            )

sizeField = \
    IntegerField('fgsize',
        default=30,
        widget=IntegerWidget(
            label='Size',description='The size of the text-entry box, in characters.',
            i18n_domain = "ploneformgen",
            label_msgid = "label_fgsize_text",
            description_msgid = "help_fgsize_text",
            ),
        )

vocabularyField = \
    LinesField('fgVocabulary',
        searchable=0,
        required=0,
        widget=LinesWidget(label='Options',
            description="""
                Use one line per option.
                Note that this may be overridden dynamically.
                [Note, you may optionally use a "value|label" format.]
                """,
            i18n_domain = "ploneformgen",
            label_msgid = "label_fgvocabulary_text",
            description_msgid = "help_fgvocabulary_text",
            ),
        )

vocabularyOverrideField = \
    TALESString('fgTVocabulary',
        schemata='overrides',
        searchable=0,
        required=0,
        validators=('talesvalidator',),
        default='',
        write_permission=EDIT_TALES_PERMISSION,
        widget=StringWidget(label="Options Vocabulary",
            description="""
                A TALES expression that will be evaluated when the form is displayed
                to get the field options.
                Leave empty if unneeded.
                Your TALES expression should evaluate as a list of (key, value) tuples.
                PLEASE NOTE: errors in the evaluation of this expression will cause
                an error on form display.
            """,
            size=70,
            i18n_domain = "ploneformgen",
            label_msgid = "label_fgtvocabulary_text",
            description_msgid = "help_fgtvocabulary_text",
            ),
        )

# establish a bare baseline
# only label field uses this without change
BareFieldSchema = ATContentTypeSchema.copy()
BareFieldSchema['title'].searchable = False
BareFieldSchema['title'].widget.label = 'Field Label'
BareFieldSchema['title'].widget.i18n_domain = "ploneformgen"
BareFieldSchema['title'].widget.label_msgid = 'label_fieldlabel_text'
BareFieldSchema['description'].searchable = False
BareFieldSchema['description'].widget.label = 'Field Help'
BareFieldSchema['description'].widget.i18n_domain = "ploneformgen"
BareFieldSchema['description'].widget.label_msgid = 'label_fieldhelp_text'
BareFieldSchema['description'].widget.description = None
BareFieldSchema['description'].widget.description_msgid = None


###
# BaseFieldSchema -- more common baseline
# Used as a base schema for several fields

BaseFieldSchema = BareFieldSchema.copy() + Schema((
        BooleanField('required',
            searchable=0,
            required=0,
            widget=BooleanWidget(
                label="Required",
                i18n_domain = "ploneformgen",
                label_msgid = "label_required_text",
                ),
            ),
        BooleanField('hidden',
            searchable=0,
            required=0,
            write_permission=EDIT_ADVANCED_PERMISSION,
            widget=BooleanWidget(
                label="Hidden",
                i18n_domain = "ploneformgen",
                label_msgid = "label_hidden_text",
                ),
            ),
        TALESString('fgTDefault',
            schemata='overrides',
            searchable=0,
            required=0,
            validators=('talesvalidator',),
            write_permission=EDIT_TALES_PERMISSION,
            default='',
            widget=StringWidget(label="Default Expression",
                description="""
                    A TALES expression that will be evaluated when the form is displayed
                    to get the field default value.
                    Leave empty if unneeded. Your expression should evaluate as a string.
                    PLEASE NOTE: errors in the evaluation of this expression will cause
                    an error on form display.
                """,
                size=70,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgtdefault_text",
                description_msgid = "help_fgtdefault_text",
                ),
            ),
        validatorOverrideField,
        TALESString('fgTEnabled',
            schemata='overrides',
            searchable=0,
            required=0,
            validators=('talesvalidator',),
            write_permission=EDIT_TALES_PERMISSION,
            default='',
            widget=StringWidget(label="Enabling Expression",
                description="""
                    A TALES expression that will be evaluated when the form is displayed
                    to determine whether or not the field is enabled.
                    Your expression should evaluate as True if
                    the field should be included in the form, False if it should be omitted.
                    Leave this expression field empty if unneeded: the field will be included. 
                    PLEASE NOTE: errors in the evaluation of this expression will cause
                    an error on form display.
                """,
                size=70,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgtenable_text",
                description_msgid = "help_fgtenable_text",
                ),
            ),
        BooleanField('serverSide',
            schemata='overrides',
            searchable=0,
            required=0,
            write_permission=EDIT_ADVANCED_PERMISSION,
            default='',
            widget=BooleanWidget(
                label="Server-Side Variable",
                description="""
                    Mark this field as a value to be injected into the
                    request form for use by action adapters and is not 
                    modifiable by or exposed to the client.
                """,
                i18n_domain = "ploneformgen",
                label_msgid = "label_server_side_text",
                ),
            ),
    ))


##
# BaseFieldSchemaStringDefault
# A base schema for fields that have a string default

BaseFieldSchemaStringDefault = BaseFieldSchema.copy() + Schema((
        StringField('fgDefault',
            searchable=0,
            required=0,
            widget=StringWidget(label='Default',
            description="""
                The value the field should contain when the form is first displayed.
                Note that this may be overridden dynamically.
            """,
            i18n_domain = "ploneformgen",
            label_msgid = "label_fgdefault_text",
            description_msgid = "help_fgdefault_text",
            ),
        ),
    ))


##
# BaseFieldSchemaLinesDefault
# A base schema for fields that have a lines default

BaseFieldSchemaLinesDefault = BaseFieldSchema.copy() + Schema((
        LinesField('fgDefault',
            searchable=0,
            required=0,
            widget=LinesWidget(label='Default',
                description="""
                    The values the field should contain when the form is first displayed.
                    Use one value per line.
                    Note that this may be overridden dynamically.
                """,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fglinesdefault_text",
                description_msgid = "help_fglinesdefault_text",
                ),
            ),
        rowsField,
        TALESString('fgTDefault',
            schemata='overrides',
            searchable=0,
            required=0,
            validators=('talesvalidator',),
            default='',
            write_permission=EDIT_TALES_PERMISSION,
            widget=StringWidget(label="Default Expression",
                description="""
                    A TALES expression that will be evaluated when the form is displayed
                    to get the field default value.
                    Leave empty if unneeded. Your expression should evaluate as a list or tuple.
                    PLEASE NOTE: errors in the evaluation of this expression will cause
                    an error on form display.
                """,
                size=70,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgtlinesdefault_text",
                description_msgid = "help_fgtlinesdefault_text",
                ),
            ),
        validatorOverrideField,
    ))


##
# BaseFieldSchemaTextDefault
# A base schema for fields that have a textarea default

BaseFieldSchemaTextDefault = BaseFieldSchema.copy() + Schema((
        TextField('fgDefault',
            searchable=0,
            required=0,
            widget=TextAreaWidget(label='Default',
                description="""
                    The text the field should contain when the form is first displayed.
                    Note that this may be overridden dynamically.
                """,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgtextdefault_text",
                description_msgid = "help_fgtextdefault_text",
                ),
            ),
        rowsField,
        maxlengthField0,
        TALESString('fgTDefault',
            schemata='overrides',
            searchable=0,
            required=0,
            validators=('talesvalidator',),
            default='',
            write_permission=EDIT_TALES_PERMISSION,
            widget=StringWidget(label="Default Expression",
                description="""
                    A TALES expression that will be evaluated when the form is displayed
                    to get the field default value.
                    Leave empty if unneeded. Your expression should evaluate as a string.
                    PLEASE NOTE: errors in the evaluation of this expression will cause
                    an error on form display.
                """,
                size=70,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgtdefault_text",
                description_msgid = "help_fgtdefault_text",
            ),
        ),
        validatorOverrideField,
    ))

##
# BaseFieldSchemaRichTextDefault
# A base schema for fields that have a rich editor default

BaseFieldSchemaRichTextDefault = BaseFieldSchema.copy() + Schema((
        TextField('fgDefault',
            searchable=0,
            required=0,
            validators = ('isTidyHtmlWithCleanup',),
            default_content_type = 'text/html',
            default_output_type = 'text/x-html-safe',
            allowable_content_types = zconf.ATDocument.allowed_content_types,
            widget=RichWidget(label='Default',
                description="""
                    The text the field should contain when the form is first displayed.
                    Note that this may be overridden dynamically.
                """,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgtextdefault_text",
                description_msgid = "help_fgtextdefault_text",
                allow_file_upload = False,
                ),
            ),
        rowsField,
        maxlengthField4k,
        TALESString('fgTDefault',
            schemata='overrides',
            searchable=0,
            required=0,
            validators=('talesvalidator',),
            default='',
            write_permission=EDIT_TALES_PERMISSION,
            widget=StringWidget(label="Default Expression",
                description="""
                    A TALES expression that will be evaluated when the form is displayed
                    to get the field default value.
                    Leave empty if unneeded. Your expression should evaluate as a string.
                    PLEASE NOTE: errors in the evaluation of this expression will cause
                    an error on form display.
                """,
                size=70,
                i18n_domain = "ploneformgen",
                label_msgid = "label_fgtdefault_text",
                description_msgid = "help_fgtdefault_text",
            ),
        ),
        validatorOverrideField,
    ))

finalizeFieldSchema(BareFieldSchema, folderish=True, moveDiscussion=False)
finalizeFieldSchema(BaseFieldSchema, folderish=True, moveDiscussion=False)
finalizeFieldSchema(BaseFieldSchemaStringDefault, folderish=True, moveDiscussion=False)
finalizeFieldSchema(BaseFieldSchemaLinesDefault, folderish=True, moveDiscussion=False)
finalizeFieldSchema(BaseFieldSchemaTextDefault, folderish=True, moveDiscussion=False)
finalizeFieldSchema(BaseFieldSchemaRichTextDefault, folderish=True, moveDiscussion=False)


class StringVocabularyField(StringField):
    """
    Parent for fields that have vocabularies.
    Overrides Vocabulary so that we can get the value from the instance
    """

    security  = ClassSecurityInfo()

    security.declarePublic('Vocabulary')
    def Vocabulary(self, content_instance=None):
        """
        Returns a DisplayList.
        """
        # if there's a TALES override, return it as a DisplayList,
        # otherwise, build the DisplayList from fgVocabulary

        fieldContainer = content_instance.findFieldObjectByName(self.__name__)

        vl = fieldContainer.getFgTVocabulary()
        if vl is not None:
            return DisplayList( data=vl )

        res = DisplayList()
        for line in fieldContainer.fgVocabulary:
            lsplit = line.split('|')
            if len(lsplit) == 2:
                key, val = lsplit
            else:
                key, val = (lsplit[0],lsplit[0])
            res.add( key, val )
        return res


class LinesVocabularyField(StringField):
    """ Same as StringVocabularyField, but based on a LinesField """

    security  = ClassSecurityInfo()

    security.declarePublic('Vocabulary')
    def Vocabulary(self, content_instance=None):
        """
        Returns a DisplayList.
        """
        # if there's a TALES override, return it as a DisplayList,
        # otherwise, build the DisplayList from fgVocabulary

        fieldContainer = content_instance.findFieldObjectByName(self.__name__)

        vl = fieldContainer.getFgTVocabulary()
        if vl is not None:
            return DisplayList( data=vl )

        res = DisplayList()
        for line in fieldContainer.fgVocabulary:
            lsplit = line.split('|')
            if len(lsplit) == 2:
                key, val = lsplit
            else:
                key, val = (lsplit[0],lsplit[0])
            res.add( key, val )
        return res


class BaseFormField(ATCTContent):
    """
    Base form generator form field class.
    Provides several methods useful to descendents.
    """
    implements(IPloneFormGenField)

    security  = ClassSecurityInfo()

    schema = BaseFieldSchema

    # Standard content type setup
    portal_type = meta_type = 'BaseFormField'
    archetype_name = 'Base Form Field'
    content_icon = 'BasicField.gif'
    typeDescription= 'A form field'

    if HAS_PLONE30:
        default_view = immediate_view = 'fg_base_view_p3'
    else:
        default_view = immediate_view = 'fg_base_view'
    suppl_views = ()

    # Let's not pollute the global "add"
    global_allow = 0


    security.declareProtected(ModifyPortalContent, 'setDescription')
    def setDescription(self, value, **kw):
        """ set description for field widget """

        self.fgField.widget.description = value
        self.getField('description').set(self, value, **kw)


    security.declareProtected(ModifyPortalContent, 'setRequired')
    def setRequired(self, value, **kw):
        """ set required flag for field """

        if type(value) == BooleanType:
            self.fgField.required = value
        else:
            self.fgField.required = value == '1' or value == 'True'


    security.declareProtected(View, 'getRequired')
    def getRequired(self, **kw):
        """ get required flag for field """

        return self.fgField.required


    security.declareProtected(ModifyPortalContent, 'setHidden')
    def setHidden(self, value, **kw):
        """ set hidden flag for field widget """

        if type(value) == BooleanType:
            if value:
                self.fgField.widget.visible = -1
            else:
                self.fgField.widget.visible = 1
        elif value == '1' or value == 'True':
            self.fgField.widget.visible = -1
        else:
            self.fgField.widget.visible = 1


    security.declareProtected(View, 'getHidden')
    def getHidden(self, **kw):
        """ get hidden flag for field widget """

        return self.fgField.widget.visible == -1

    security.declareProtected(View, 'getServerSide')
    def getServerSide(self, **kw):
        """ return server side flag for field """
        try:
            return self.getField('serverSide').get(self)
        except AttributeError:
            return False

    security.declareProtected(ModifyPortalContent, 'setTitle')
    def setTitle(self, value, **kw):
        """ set title of object and field label """

        self.title = value
        if isinstance(value, unicode):
            uvalue = value
        else:
            utils = getToolByName(self, 'plone_utils')
            charset = utils.getSiteEncoding()
            uvalue = unicode(value, charset)
        self.fgField.widget.label = uvalue


    security.declareProtected(ModifyPortalContent, 'setId')
    def setId(self, value):
        """Sets the object id. Changes both object and field id.
        """

        badIds = (
            'language',
            'form',
            'form_submit',
            'fieldset',
            'last_referer',
            'add_reference',
            )

        if value in badIds:
            raise BadRequest, 'The id "%s" is reserved.' % value

        BaseContent.setId(self, value)
        self.fgField.__name__ = self.getId()


    security.declareProtected(ModifyPortalContent, 'setFgmaxlength')
    def setFgmaxlength(self, value, **kw):
        """ set maxlength for field widget """

        if value == '0':
            self.fgField.widget.maxlength = False
        else:
            self.fgField.widget.maxlength = value


    security.declareProtected(View, 'getFgmaxlength')
    def getFgmaxlength(self, **kw):
        """ get maxlength for field widget """

        ml = self.fgField.widget.maxlength
        if ml:
            return ml
        else:
            return '0'


    security.declareProtected(ModifyPortalContent, 'setFgsize')
    def setFgsize(self, value, **kw):
        """ set size for field widget """

        self.fgField.widget.size = value


    security.declareProtected(View, 'getFgsize')
    def getFgsize(self, **kw):
        """ get size for field widget """

        return self.fgField.widget.size

    security.declareProtected(ModifyPortalContent, 'setFgRows')
    def setFgRows(self, value, **kw):
        """ sets widget rows """

        self.fgField.widget.rows = value


    security.declareProtected(View, 'getFgRows')
    def getFgRows(self, **kw):
        """ gets widget rows """

        return self.fgField.widget.rows


# style sheet sets visual column widths
#    security.declareProtected(View, 'setFgCols')
#    def setFgCols(self, value, **kw):
#        """ sets widget columns """
#
#        self.fgField.widget.cols = value
#
#
#    security.declareProtected(View, 'getFgCols')
#    def getFgCols(self, **kw):
#        """ Sets widget columns """
#
#        return self.fgField.widget.cols


    security.declareProtected(View, 'getMinval')
    def getMinval(self, **kw):
        """ get minval for field """

        return self.fgField.minval


    security.declareProtected(ModifyPortalContent, 'setMinval')
    def setMinval(self, value, **kw):
        """ set minval for field """

        self.fgField.minval = value


    security.declareProtected(View, 'getMaxval')
    def getMaxval(self, **kw):
        """ get maxval for field """

        return self.fgField.maxval


    security.declareProtected(ModifyPortalContent, 'setMaxval')
    def setMaxval(self, value, **kw):
        """ set maxaxval for field """

        self.fgField.maxval = value


    security.declareProtected(ModifyPortalContent, 'setFgStringValidator')
    def setFgStringValidator(self, value, **kw):
        """ set simple validator """

        if value:
            self.fgField.validators = ('isNotTooLong', value)
        else:
            self.fgField.validators = ('isNotTooLong',)
        self.fgField._validationLayer()

        self.fgStringValidator = value


    security.declareProtected(View, 'fgPrimeDefaults')
    def fgPrimeDefaults(self, request, contextObject=None):
        """ primes request with default """

        # the field macros will try to get the field value
        # via Field.getEditAccessor. Unfortunately, it looks for it
        # as an attribute of the object, not the field.
        # so, communicate via the request, but don't overwrite
        # what's already there.

        if safe_hasattr(self, 'getFgTDefault') and self.getRawFgTDefault():
            if contextObject:
                # see note in fgvalidate
                value = self.getFgTDefault(expression_context=getExprContext(self, contextObject))
            else:
                value = self.getFgTDefault()
        else:
            value = None

        if (value is None) and safe_hasattr(self, 'getFgDefault'):
            value = self.getFgDefault()
        if value:
            request.form.setdefault(self.fgField.__name__, value)


    security.declareProtected(View, 'fgFields')
    def fgFields(self, request=None):
        """ generate fields on the fly; also primes request with defaults """

        if request:
            self.fgPrimeDefaults(request, contextObject=self.aq_parent)

        return ( self.fgField, )


    security.declarePrivate('_specialValidator')
    def specialValidator(self, value, field, REQUEST, errors):
        """
        Override this method to provide a custom field validator.
        It will be called before other validators.
        As with Archetypes validators, return 0 if value is OK,
        a string error message if invalid.
        If invalid, no other validators will be run for this field.
        """

        return 0


    security.declareProtected(View, 'fgvalidate')
    def fgvalidate(self, REQUEST=None, errors=None, data=None, metadata=None):
        """Validates the field data from the request.
        """

        _marker = []
        if errors is None:
            errors = {}
        if errors:
            return errors

        field = self.fgField

        result = field.widget.process_form(self, field, REQUEST.form, empty_marker=_marker)

        if result is None or result is _marker:
            #XXX Make this smarter
            value = ''
        else:
            value = result[0]

        # workaround what I consider a Zope marshalling error: the production
        # of lists like ['one', ''] and [''].
        # no need to worry about polymorphism here, as this is a very particular
        # case.
        if isinstance(value, type([])) and len(value) and \
            (type(value[-1]) in StringTypes) and (len(value[-1]) == 0):
            value.pop()

        # eliminate trailing white space in string types.
        if safe_hasattr(value, 'rstrip'):
            newvalue = value.rstrip()
            if newvalue != value:
                value = newvalue
                # since strings are immutable, we have to manually store it back to the request
                if safe_hasattr(REQUEST, 'form'):
                    REQUEST.form[self.getFieldFormName()] = value                        

        # Archetypes field validation
        res = field.validate(instance=self,
                            value=value,
                            errors=errors,
                            REQUEST=REQUEST)
        
        if not res:
            # give the field itself an opportunity to validate.
            res = self.specialValidator(value, field, REQUEST, errors)

        if res:
            errors[field.getName()] = validationMessages.cleanupMessage(res, self.REQUEST, self)
        elif safe_hasattr(self, 'getFgTValidator') and self.getRawFgTValidator():
            # process the override validator TALES expression

            # create a context for expression evaluation.
            # Note that we're explicitly passing the form object to getExprContext;
            # this makes sure we don't cache the wrong context.
            context = getExprContext(self, self.aq_parent)

            # put this field's input (from request) into the context globals
            # as 'value'
            context.setGlobal('value', REQUEST.form.get(self.getFieldFormName(), None))

            # call the tales expression, passing our custom context
            cerr = self.getFgTValidator(expression_context=context)
            if cerr:
                errors[field.getName()] = cerr

        return errors


    security.declareProtected(View, 'fgGetSuccessAction')
    def fgGetSuccessAction(self):
        """
             Returns string id of success action template or script.
             Controller will traverse to this on successful validation.
         """

        return 'traverse_to:string:fg_result_view'


    security.declareProtected(View, 'isLabel')
    def isLabel(self):
        """Returns True if this field is a label and should only be
           shown in the edit form.
        """

        # override this method when field really is a label

        return False


    security.declareProtected(View, 'isFileField')
    def isFileField(self):
        """Returns True if the embedded field acts like a file field
        """

        # override if field really is file-like

        return False


    security.declareProtected(View, 'getFormPrologue')
    def getFormPrologue(self, **kw):
        """ Returns empty string to prevent display of prologue on field view """
        return ""


    security.declareProtected(View, 'getFormEpilogue')
    def getFormEpilogue(self, **kw):
        """ Returns empty string to prevent display of epilogue on field view """
        return ""


    security.declareProtected(View, 'htmlValue')
    def htmlValue(self, REQUEST):
        """ return from REQUEST, this field's value, rendered as XHTML.
        """

        # override this if a field's value from request isn't suitable for display
        # or is already in HTML

        value = REQUEST.form.get(self.__name__, 'No Input')
        valueType = type(value)
        value = str(value)

        # eliminate square brackets around lists --
        # they mean nothing to end users
        if (valueType == type([])):
            value = value[1:-1]

        if value.find('\n') >= 0:
            # strings with embedded eols will benefit from the full
            # transform
            pt = getToolByName(self, 'portal_transforms')
            s = pt.convert('text_to_html', value).getData()
            # let's return something more semantically neutral than
            # a paragraph.
            return s.replace('<p>', '<div>').replace('</p>', '</div>')
        else:
            # don't bother with the overhead of a portal transform
            return cgi.escape(value)


    # The Archetypes file.pt macro tries -- for reasons I don't
    # really understand -- to call the field's accessor.
    # So, let's supply this as a harmless one when necessary.
    def nullAccessor(self, **kwargs):
        return None


    security.declarePublic('getFieldFormName')
    def getFieldFormName(self):
        """Returns field name as used in form and request.form.
           To be overridden when form field name varies from the field name,
           as, for example, with file field."""

        return self.fgField.getName()


    def manage_afterAdd(self, item, container):
        # TODO: when we're done with 2.1.x, implement this via event subscription

        ATCTContent.manage_afterAdd(self, item, container)

        id = self.getId()
        if self.fgField.__name__ != id:
            self.fgField.__name__ = id


    def processForm(self, data=1, metadata=0, REQUEST=None, values=None):
        # override base so that we can selectively redirect back to the form
        # rather than to the field view.

        # base processing
        ATCTContent.processForm(self, data, metadata, REQUEST, values)
        
        # if the referer is the item itself, let nature take its course;
        # if not, redirect to form after a commit.
        referer = self.REQUEST.form.get('last_referer', None)
        if referer is not None and referer.split('/')[-1] != self.getId():
            transaction.commit()
            raise zExceptions.Redirect, "%s?qedit=1" % self.formFolderObject().absolute_url()



"""RichLabelWidget -- displays formatted text in edit form"""

# This widget isn't anything more than a placeholder. Unmodified Archetypes macros
# won't allow for a field without a label. So, to minimize intervention, this is
# handled in fg_edit_macros

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget

# The widget itself

class RichLabelWidget(TypesWidget):
    """This widget displays formatted text inside a form.
    It is probably useless outside PloneForm, since the body
    needs to be set outside the field.
    """

    # Use the base class properties, and add two of our own
    _properties = TypesWidget._properties.copy()
    _properties.update({'macro' : 'widget_richlabel',
                        },)

# Register the widget with Archetypes
registerWidget(RichLabelWidget,
               title = 'Rich label widget',
               description= ('Renders formatted text inside form',),
               used_for = ('Products.Archetypes.Field.BooleanField',)
               )



from Acquisition import aq_inner
from Products.Five import BrowserView
from Products.PloneFormGen import HAS_PLONE30

DEFAULT_SUBMISSION_MARKER = 'form.submitted'

"""
Cases that won't work / need testing or thinking through
- success override that uses a traverse_to action (actually theoretically valid,
  as long as the thing being traversed to doesn't pull in main_template)
- do we need to buffer anything else besides the submission marker and
  controller state?
"""

class EmbeddedPFGView(BrowserView):
    """ Browser view that can update and render a PFG form in some other context
    """
    
    # Prefix to ensure that we don't try to process other forms that might
    # be on the same page
    prefix = u''
    
    # optional form action override
    action = None
    
    # enable form unload warning if it has been edited?
    enable_unload_protection = False
    
    # auto-focus the form on page load?
    enable_auto_focus = False
    
    def setPrefix(self, prefix):
        self.prefix = prefix
        
    def setAction(self, action):
        self.action = action
    
    def __call__(self):
        
        if self.prefix:
            form_marker = self.prefix + '.' + DEFAULT_SUBMISSION_MARKER
        else:
            form_marker = DEFAULT_SUBMISSION_MARKER

        # CMFFormController form processing is based on the presence of a 'form.submitted'
        # key in the request.  We need to translate our prefixed version.
        if DEFAULT_SUBMISSION_MARKER in self.request.form:
            fiddled_submission_marker = self.request.form[DEFAULT_SUBMISSION_MARKER]
        else:
            fiddled_submission_marker = None
        
        # the form.submitted marker gets removed by CMFFormController, but we
        # need to be able to test for it from our own template as well
        self.request.other['pfg_form_submitted'] = False
        
        if self.request.environ.get('X_PFG_RETRY', False):
            # We check for the absence of the X_PFG_RETRY flag in the request,
            # to avoid processing the form in the edge case where the form already completed
            # and is using a Retry exception to traverse to the thank you page, but the thank
            # you page also has the same embedded form on it somewhere
            if DEFAULT_SUBMISSION_MARKER in self.request.form:
                del self.request.form[DEFAULT_SUBMISSION_MARKER]
        elif form_marker in self.request.form:
            self.request.form[DEFAULT_SUBMISSION_MARKER] = True
            self.request.other['pfg_form_submitted'] = True
        elif self.prefix and DEFAULT_SUBMISSION_MARKER in self.request.form:
            # not our form; temporarily remove the form.submitted key to avoid a false positive
            del self.request.form[DEFAULT_SUBMISSION_MARKER]
        
        # And we need to pass the form marker in the request so it can be inserted
        # into the form (we can't just use it as an argument to the controller template,
        # b/c then it won't survive validation)
        self.request.form['pfg_form_marker'] = form_marker
        
        # temporarily clear out the controller_state from the request in case we're embedded in
        # another controller page template
        fiddled_controller_state = self.request.get('controller_state', None)
        self.request.set('controller_state', None)
        
        # pass the form action override
        # (we do this in the request instead of passing it in when calling the .cpt, because
        # the .cpt might end up getting called again after validation or something)
        if self.action is not None:
            self.request.set('pfg_form_action', self.action)
        else:
            self.request.set('pfg_form_action', self.request['URL'])
        
        # Delegate to CMFFormController page template so we can share logic with the standalone form
        try:
            context = aq_inner(self.context)
            if HAS_PLONE30:
                return context.fg_embedded_view_p3(
                    enable_unload_protection=self.enable_unload_protection,
                    enable_auto_focus=self.enable_auto_focus
                    )
            else:
                return context.fg_embedded_view()
        finally:
            # Clean up
            if fiddled_submission_marker is not None:
                self.request.form['form.submitted'] = fiddled_submission_marker
            elif fiddled_submission_marker is None and 'form.submitted' in self.request.form:
                del self.request.form['form.submitted']
            if fiddled_controller_state is not None:
                self.request.set('controller_state', fiddled_controller_state)
            del self.request.form['pfg_form_marker']
            del self.request.other['pfg_form_action']
            del self.request.other['pfg_form_submitted']



from Products.validation import validation, interfaces
from zope.component import getMultiAdapter
from Acquisition import aq_inner

class CaptchaValidator:
    """ Archetypes field validator for use with collective.captcha or collective.recaptcha
    """

    __implements__ = (interfaces.ivalidator,)

    name = 'CaptchaValidator'

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        
        context = kwargs.get('instance')
        request = kwargs.get('REQUEST')
        captcha = getMultiAdapter((context, request), name='captcha')
        if not captcha.verify(value):
            return ("Verification failed: Please type the characters you see below.")
        return 1

validation.register(CaptchaValidator('isCorrectCaptcha'))


from zope.interface import implements
from zope.formlib import form
from zope.component import getMultiAdapter

from Products.Five import BrowserView
from Products.Five.formlib import formbase

from Products.statusmessages.interfaces import IStatusMessage

from Products.PloneFormGen import interfaces
from Products.PloneFormGen import PloneFormGenMessageFactory as _

from Products.GenericSetup.context import TarballExportContext, TarballImportContext
from Products.GenericSetup.interfaces import IFilesystemExporter, IFilesystemImporter

class FormFolderExportView(BrowserView):
    """See ..interfaces.exportimport.IFormFolderExportView
    """
    implements(interfaces.IFormFolderExportView)
    
    def __call__(self):
        """See ..interfaces.exportimport.IFormFolderExportView.__call__
        """
        ctx = TarballExportContext(self.context)
        
        self.request.RESPONSE.setHeader('Content-type', 'application/x-gzip')
        self.request.RESPONSE.setHeader('Content-disposition',
                           'attachment; filename=%s' % ctx.getArchiveFilename())
        
        # export the structure treating the current form as our root context
        IFilesystemExporter(self.context).export(ctx, 'structure', True)
        
        return ctx.getArchive()
    

class FormFolderImportView(formbase.Form):
    """The formlib class for importing of exported PFG form folders
    """
    implements(interfaces.IFormFolderImportView)
    
    form_fields = form.Fields(interfaces.IImportSchema)
    status = errors = None
    prefix = 'form'
    
    @form.action(_(u"import"))
    def action_import(self, action, data):
        if data.get('purge', False) == True:
            # user has requested removal of existing fields
            self.context.manage_delObjects(ids=self.context.objectIds())
        
        ctx = TarballImportContext(self.context, data['upload'])
        IFilesystemImporter(self.context).import_(ctx, 'structure', True)
        
        message = _(u'Form imported.')
        IStatusMessage(self.request).addStatusMessage(message, type='info')
        
        url = getMultiAdapter((self.context, self.request), name='absolute_url')()
        self.request.response.redirect(url)
        
        return ''
    




#
# Integration tests for interaction with GenericSetup infrastructure
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from zope.interface.verify import verifyObject, verifyClass
from zope.component import getMultiAdapter

from Products.PloneFormGen.tests import pfgtc
from Products.PloneFormGen import interfaces
from Products.PloneFormGen.browser import exportimport
from Products.PloneFormGen import content


class TestFormGenInterfaces(pfgtc.PloneFormGenTestCase):
    """ Some boilerplate-ish tests to confirm that that classes
        and instances confirm to the interface contracts intended.
    """
    def afterSetUp(self):
        pfgtc.PloneFormGenTestCase.afterSetUp(self)
        
        # add form folder for use in tests
        self.folder.invokeFactory('FormFolder', 'ff1')
    
    def testBrowserViewClassInterfaces(self):
        """Some basic boiler plate testing of interfaces and classes"""
        # verify IFormFolderExportView
        self.failUnless(interfaces.IFormFolderExportView.implementedBy(exportimport.FormFolderExportView))
        self.failUnless(verifyClass(interfaces.IFormFolderExportView, exportimport.FormFolderExportView))
    
    def testBrowserViewObjectsVerify(self):
        # verify views are objects of the expected class, verified implementation
        form_folder_export = getMultiAdapter((self.folder.ff1, self.app.REQUEST), name='export-form-folder')
        self.failUnless(isinstance(form_folder_export, exportimport.FormFolderExportView))
        self.failUnless(verifyObject(interfaces.IFormFolderExportView, form_folder_export))
    
    def testContentClassInterfaces(self):
        self.failUnless(interfaces.IPloneFormGenFieldset.implementedBy(content.fieldset.FieldsetFolder))
        self.failUnless(verifyClass(interfaces.IPloneFormGenFieldset, content.fieldset.FieldsetFolder))
    

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestFormGenInterfaces))
    return suite



"""actionAdapter -- A base adapter for form actions"""

__author__  = 'Steve McMahon <steve@dcn.org>'
__docformat__ = 'plaintext'

import re

from zope.interface import implements
import transaction
import zExceptions
from AccessControl import ClassSecurityInfo

from Products.CMFCore.permissions import View, ModifyPortalContent

from Products.Archetypes.public import *
from Products.Archetypes.utils import shasattr

from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.base import registerATCT

from Products.TALESField import TALESString

from Products.PloneFormGen.config import *
from Products.PloneFormGen import HAS_PLONE30
from Products.PloneFormGen.interfaces import IPloneFormGenActionAdapter

FormAdapterSchema = ATContentTypeSchema.copy() + Schema((
    TALESString('execCondition',
        schemata='overrides',
        searchable=0,
        required=0,
        validators=('talesvalidator',),
        default='',
        write_permission=EDIT_TALES_PERMISSION,
        read_permission=ModifyPortalContent,
        isMetadata=True, # just to hide from base view
        widget=StringWidget(label="Execution Condition",
            description="""
                A TALES expression that will be evaluated to determine whether or not
                to execute this action.
                Leave empty if unneeded, and the ection will be executed. 
                Your expression should evaluate as a boolean; return True if you wish
                the action to execute.
                PLEASE NOTE: errors in the evaluation of this expression will cause
                an error on form display.
            """,
            size=70,
            i18n_domain = "ploneformgen",
            label_msgid = "label_execcondition_text",
            description_msgid = "help_execcondition_text",
        ),
    ),
    ))

if not HAS_PLONE30:
    FormAdapterSchema['description'].schemata = 'metadata'
finalizeATCTSchema(FormAdapterSchema, folderish=True, moveDiscussion=False)

if HAS_PLONE30:
    # avoid showing unnecessary schema tabs
    for afield in ('description',
                   'subject', 
                   'relatedItems', 
                   'location', 
                   'language', 
                   'effectiveDate', 
                   'expirationDate', 
                   'creation_date', 
                   'modification_date', 
                   'creators', 
                   'contributors', 
                   'rights', 
                   'allowDiscussion', 
                   'excludeFromNav', ):
        FormAdapterSchema[afield].widget.visible = {'view':'invisible','edit':'invisible'}
        FormAdapterSchema[afield].schemata = 'default'


class FormActionAdapter(ATCTContent):
    """A base action adapter"""

    implements(IPloneFormGenActionAdapter)

    schema         =  FormAdapterSchema

    content_icon   = 'FormAction.gif'
    meta_type      = 'FormActionAdapter'
    portal_type    = 'FormActionAdapter'
    archetype_name = 'Form Action Adapter'

    immediate_view = 'base_view'
    default_view   = 'base_view'
    suppl_views = ()

    typeDescription= 'An adapter that supplies a form action.'

    global_allow = 0    

    security       = ClassSecurityInfo()


    def onSuccess(self, fields, REQUEST=None):
        """ Called by form to invoke custom success processing.
            return None (or don't use "return" at all) if processing is
            error-free.
            
            Return a dictionary like {'field_id':'Error Message'}
            and PFG will stop processing action adapters and
            return back to the form to display your error messages
            for the matching field(s).

            You may also use Products.PloneFormGen.config.FORM_ERROR_MARKER
            as a marker for a message to replace the top-of-the-form error
            message.

            For example, to set a message for the whole form, but not an
            individual field:

            {FORM_ERROR_MARKER:'Yuck! You will need to submit again.'}

            For both a field and form error:

            {FORM_ERROR_MARKER:'Yuck! You will need to submit again.',
             'field_id':'Error Message for field.'}
            
            Messages may be string types or zope.i18nmessageid objects.                
        """
        
        # fields will be a sequence of objects with an IPloneFormGenField interface
        
        pass


    def at_post_create_script(self):
        """ activate action adapter in parent folder """
        
        # XXX TODO - change to use events when we give up on Plone 2.1.x
        
        ATCTContent.at_post_create_script(self)

        self.aq_parent.addActionAdapter(self.id)


    def processForm(self, data=1, metadata=0, REQUEST=None, values=None):
        # override base so that we can selectively redirect back to the form
        # rather than to the adapter view.

        # base processing
        ATCTContent.processForm(self, data, metadata, REQUEST, values)

        # if the referer is the item itself, let nature take its course;
        # if not, redirect to form after a commit.
        referer = self.REQUEST.form.get('last_referer', None)
        if referer is not None and referer.split('/')[-1] != self.getId():
            transaction.commit()
            raise zExceptions.Redirect, "%s?qedit=1" % self.formFolderObject().absolute_url()



# Make sure this directory is treated as a module

from richlabel import RichLabelWidget
from fieldset import FieldsetStartWidget, FieldsetEndWidget
from likertWidget import LikertWidget
from captcha import CaptchaWidget



from zope.interface import Interface
from zope.schema import Bytes, Bool

from Products.PloneFormGen import PloneFormGenMessageFactory as _

class IFormFolderExportView(Interface):
    """ Browser view registered for IPloneFormGenForm
        which can be called when requesting a placeful
        export of the give form's configuration.
    """
    def __call__():
        """ Returns export data to the client
        """

class IImportSchema(Interface):
    """Schema for form import.
    """
    upload = Bytes(
        title=_(u'Upload'),
        required=True)
    
    purge = Bool(
        title=_(u'Remove Existing Form Items?'),
        default=False,
        required=False)
    

class IFormFolderImportView(Interface):
    """Interface for import form
    """



__author__ = """Titus Anderson <titus.anderson@louisville.edu>"""
__docformat__ = 'plaintext'

# Copyright (c) 2007 by Copyright (c) 2007 Titus Anderson
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from Products.Archetypes.public import *
from Products.Archetypes.utils import shasattr

from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.configuration import zconf

from Products.Archetypes.Field import ObjectField

from Products.Archetypes.Registry import registerField
from Products.PloneFormGen.widgets import LikertWidget

from Products.PloneFormGen import PloneFormGenMessageFactory as _

from AccessControl import ClassSecurityInfo


class LikertField(ObjectField):
    __implements__ = (getattr(ObjectField, '__implements__', ()),)

    _properties = ObjectField._properties.copy()
    _properties.update({
        'type' : 'likert',
        'widget' : LikertWidget(),
        'answerSet' : (),
        'questionSet' : (),
    })

    security = ClassSecurityInfo()

    security.declarePublic('get')
    def get(self, instance, **kwargs):
        """ Return LikertField Data
            Result is a tuple of responses
        """

        value = ObjectField.get(self, instance, **kwargs)
        if value is None:
            return tuple([None] * len(questions))
        else:
            return value

    security.declarePublic('set')
    def set(self, instance, value, **kwargs):
        """ Set the LikertField data """

        if type(value) in STRING_TYPES:
            value = [v.strip() for v in value.split(',')]
        ObjectField.set(self, instance, value, **kwargs)

    security.declarePublic('getQuestionSet')
    def getQuestionSet(self):
        return self.questionSet

    security.declarePublic('getAnswerSet')
    def getAnswerSet(self):
        return self.answerSet

    def validate_required(self, instance, value, errors):
        for index in range(self.questionSet):
            if not value[index]:
                label = self.widget.Label(instance)
                name = self.getName()
                error = 'Answers to all questions of %s are required, please correct' % label
                errors[name] = error
                return error
        return None

    def validate(self, value, instance, errors=None, **kwargs):
        error = _(u'pfg_allRequired', u'An answer is required for each question.')
        if not self.required:
            return None
        for index in range(len(self.questionSet)):
            if (index > len(value)) or not value[index]:
                fname = self.getName()
                if fname not in errors:
                    errors[fname] = error
                return error
        return None

registerField(LikertField,
              title='Likert Field',
              description='Used for collecting Likert survey answers'
)



import unittest
import doctest

from Testing import ZopeTestCase as ztc
from Products.PloneFormGen.tests.pfgtc import PloneFormGenFunctionalTestCase

testfiles = (
    'browser.txt',
    'ssl.txt',
    'serverside_field.txt',
    '../dollarReplace.py',
)

def test_suite():
    return unittest.TestSuite([

        ztc.FunctionalDocFileSuite(
            f, package='Products.PloneFormGen.tests',
            test_class=PloneFormGenFunctionalTestCase,
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE | doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS)
        
            for f in testfiles
        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#
# Test PloneFormGen event-handler functionality
#

import os, sys

from Products.PloneFormGen.tests import pfgtc

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

        
class TestAdapterPaste(pfgtc.PloneFormGenTestCase):
    """Ensure content types can be created and edited"""

    adapterTypes = (
        'FormSaveDataAdapter',
        'FormMailerAdapter',
        'FormCustomScriptAdapter',
    )

    def afterSetUp(self):
        pfgtc.PloneFormGenTestCase.afterSetUp(self)
        self.folder.invokeFactory('FormFolder', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
    
    def testPastedAdaptersAreActive(self):
        self.loginAsPortalOwner() # creating a FormCustomScriptAdapter requires elevated permissions
        for f in self.adapterTypes:
            fname = "%s1" % f
            self.ff1.invokeFactory(f, fname)
            f1 = getattr(self.ff1, fname)
            copy = self.ff1.manage_copyObjects(fname)
            new_id = self.ff1.manage_pasteObjects(copy)[0]['new_id']
            
            self.failUnless(new_id in self.ff1.getActionAdapter())
    
    def testActiveAdaptersNotDuplicatedOnFormCopy(self):
        self.loginAsPortalOwner()
        copy = self.folder.manage_copyObjects('ff1')
        new_id = self.folder.manage_pasteObjects(copy)[0]['new_id']
        ff2 = getattr(self.folder, new_id)
        active_adapters = ff2.getActionAdapter()
        self.assertEqual(active_adapters, ('mailer',))
    
    def testTempFactoryAdaptersNotActivatedOnForm(self):
        self.loginAsPortalOwner() # creating a FormCustomScriptAdapter requires elevated permissions
        adapter_count = len(self.ff1.getActionAdapter())
        for f in self.adapterTypes:
            temp_adapter = self.ff1.restrictedTraverse('portal_factory/%s/%s_tmp' % (f,f))
        # temporary objects shouldn't be active on the form folder
        self.assertEqual(adapter_count, len(self.ff1.getActionAdapter()))
        

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestAdapterPaste))
    return suite



from csv import reader
from csv import writer
from ConfigParser import ConfigParser
from StringIO import StringIO

from zope.interface import implements

from Products.GenericSetup.interfaces import IFilesystemExporter
from Products.GenericSetup.interfaces import IFilesystemImporter
from Products.CMFCore.exportimport.content import StructureFolderWalkingAdapter

from Products.CMFCore.utils import getToolByName

from Products.Archetypes.utils import mapply
#
#   Filesystem export/import adapters
#
class FormFolderWalkingAdapter(StructureFolderWalkingAdapter):
    """ 
    """
    implements(IFilesystemExporter, IFilesystemImporter)
    
    def export(self, export_context, subdir, root=False):
        """ See IFilesystemExporter.
        """
        # Enumerate exportable children
        exportable = self.context.contentItems()
        exportable = [x + (IFilesystemExporter(x, None),) for x in exportable]
        exportable = [x for x in exportable if x[1] is not None]
        
        stream = StringIO()
        csv_writer = writer(stream)
        
        for object_id, object, ignored in exportable:
            csv_writer.writerow((object_id, object.getPortalTypeName()))
        
        if not root:
            subdir = '%s/%s' % (subdir, self.context.getId())
        
        export_context.writeDataFile('.objects',
                                     text=stream.getvalue(),
                                     content_type='text/comma-separated-values',
                                     subdir=subdir,
                                    )
        
        marshaller = self.context.Schema().getLayerImpl('marshall')
        ctype, length, got = marshaller.marshall(self.context)
        
        export_context.writeDataFile('.properties',
                                    text=got,
                                    content_type=ctype,
                                    subdir=subdir,
                                    )
        
        for id, object in self.context.objectItems():
            
            adapter = IFilesystemExporter(object, None)
            
            if adapter is not None:
                adapter.export(export_context, subdir)
    
    def _makeInstance(self, id, portal_type, subdir, import_context):
        context = self.context
        properties = import_context.readDataFile('.properties',
                                                 '%s/%s' % (subdir, id))
        tool = getToolByName(context, 'portal_types')
        
        try:
            tool.constructContent(portal_type, context, id)
        except ValueError: # invalid type
            return None
        
        content = context._getOb(id)
        
        if properties is not None:
            # Marshall the data
            marshaller = content.Schema().getLayerImpl('marshall')
            marshaller.demarshall(content, properties)
        
        return content
    




#
# Integration tests for interaction with GenericSetup infrastructure
#

import os, sys
import re
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from tarfile import TarFile
from ZPublisher.HTTPRequest import FileUpload
from cgi import FieldStorage

from transaction import commit
from StringIO import StringIO

from zope.component import getMultiAdapter

from Products.Five import zcml
from Products.Five import fiveconfigure

from Products.GenericSetup.tests.common import DummyExportContext
from Products.GenericSetup.tests.common import TarballTester

from Products.PloneFormGen import HAS_PLONE30
from Products.PloneFormGen.tests import pfgtc

from Testing import ZopeTestCase
from Products.PloneTestCase.layer import PloneSite

zcml_string = """\
<configure xmlns="http://namespaces.zope.org/zope"
           xmlns:gs="http://namespaces.zope.org/genericsetup"
           package="Products.PloneFormGen">

    <gs:registerProfile
        name="testing"
        title="PloneFormGen testing"
        description="Used for testing only" 
        directory="tests/profiles/testing"
        for="Products.CMFPlone.interfaces.IPloneSiteRoot"
        provides="Products.GenericSetup.interfaces.EXTENSION"
        />
        
</configure>
"""

class TestFormGenGSLayer(PloneSite):
    @classmethod
    def setUp(cls):
        fiveconfigure.debug_mode = True
        zcml.load_string(zcml_string)
        fiveconfigure.debug_mode = False
        
        app = ZopeTestCase.app()
        portal = app.plone
        
        # elevate permissions
        from AccessControl.SecurityManagement import newSecurityManager, noSecurityManager
        user = portal.getWrappedOwner()
        newSecurityManager(None, user)
        
        portal_setup = portal.portal_setup
        
        if HAS_PLONE30:
            portal_setup.runAllImportStepsFromProfile('profile-Products.PloneFormGen:testing')
        else:
            # BBB: remove conditional once PFG no longer supports 2.5.x
            old_context = portal_setup.getImportContextID()

            # run testing install process
            portal_setup.setImportContext('profile-Products.PloneFormGen:testing')
            portal_setup.runAllImportSteps()

            portal_setup.setImportContext(old_context)
            
        
        # drop elevated perms
        noSecurityManager()
        
        commit()
        ZopeTestCase.close(app)

class ExportImportTester(pfgtc.PloneFormGenTestCase, TarballTester):
    """Base class for integration test suite for export/import """
    
    layer = TestFormGenGSLayer
    
    def _makeForm(self):
        self.folder.invokeFactory('FormFolder', 'ff1')
        self.ff1 = getattr(self.folder, 'ff1')
    
    def _prepareFormTarball(self):
        """we could use our @@export-form-folder view, 
           but we want a more atomic test, so we make
           a tarfile for our test.  the approach to making
           a tarfile is a bit strange, but does the job
        """
        in_fname = "test_form_1_form-folder.tar.gz"
        test_dir = os.path.dirname(__file__)
        
        def _add_form_structure_to_archive(archive):
            form_relative_path = os.path.join("profiles", "testing", "structure",
                "Members", "test_user_1_", "test_form_1_form-folder")
            abs_path = os.path.join(test_dir, form_relative_path)
            
            # add structure folder
            os.chdir(os.path.join(test_dir, "profiles", "testing"))
            archive.add("structure", recursive=False)
            
            for f in os.listdir(abs_path):
                os.chdir(abs_path) # add form data w/o full directory tree
                archive.add(f, arcname=os.path.join("structure", f))
        
        # make me a tarfile in the current dir
        os.chdir(test_dir)
        archive = TarFile.open(name=in_fname, mode='w:gz')
        _add_form_structure_to_archive(archive)
        archive.close()
        
        # get it and upload
        in_file = open(os.path.join(test_dir, in_fname))
        env = {'REQUEST_METHOD':'PUT'}
        headers = {'content-type':'text/html',
                   'content-length': len(in_file.read()),
                   'content-disposition':'attachment; filename=%s' % in_file.name}
        in_file.seek(0)
        fs = FieldStorage(fp=in_file, environ=env, headers=headers)
        return FileUpload(fs)
    
    def _extractFieldValue(self, setval):
        """differentiate datastructure for TALESField instances vs.
           standard ATField instances
        """
        if hasattr(setval, 'raw'):
            setval = setval.raw
        
        return setval
    
    def _verifyProfileFormSettings(self, form_ctx):
        form_values = {
            'title':'My OOTB Form',
            #'isDiscussable':False,
            'description':'The description for our OOTB form',
            # XXX andrewb enable when form props are supported
            # 'submitLabel':'Hit Me',
        }
        
        for k,v in form_values.items():
            self.assertEqual(v, self._extractFieldValue(form_ctx[k]),
                "Expected '%s' for field %s, Got '%s'" % (v, k, form_ctx[k]))
    
    def _verifyFormStockFields(self, form_ctx, purge):
        """ helper method to verify adherence to profile-based
            form folder in tests/profiles/testing directory
        """
        form_field_ids = form_ctx.objectIds()
        for form_field in ('comments', 'replyto', 'topic', 'mailer', 'thank-you'):
            if purge:
                self.failIf(form_field in form_field_ids, 
                    "%s unexpectedly found in %s" % (form_field, form_ctx.getId()))
                continue
            
            self.failUnless(form_field in form_field_ids,
                "%s not found in %s" % (form_field, form_ctx.getId()))
    
    def _verifyProfileForm(self, form_ctx, form_fields=None):
        """ helper method to verify adherence to profile-based
            form folder in tests/profiles/testing directory
        """
        form_id_prefix = 'test_form_1_'
        
        if not form_fields:
            form_fields = [
               {
                'id':'%sreplyto' % form_id_prefix,
                'title':'Test Form Your E-Mail Address',
                'required':True,
                #'isDiscussable':False,
                'fgTDefault':'here/memberEmail',},
               {
                'id':'%shidden' % form_id_prefix,
                'title':'This is a sample hidden field',
                'required':False,
                #'isDiscussable':False,
                'hidden':True,},
               {
                'id':'%sfieldset-folder' % form_id_prefix,
                'title':'Fields grouped in a fieldset',
                'useLegend':False,
                'subfields':[{
                    'id':'%shidden_fieldset' % form_id_prefix,
                    'title':'This is a sample hidden field fieldset',
                    'required':True,
                    #'isDiscussable':False,
                    'hidden':True,
                },],},
               {
                'id':'%stopic' % form_id_prefix,
                'title':'Test Form Subject',
                # 'isDiscussable':False,
                'required':True,},
               {
                'id':'%scomments' % form_id_prefix,
                'fgDefault': 'string:Test Comment',},
            ]
        # get our forms children to ensure proper config
        for form_field in form_fields:
            self.failUnless('%s' % form_field['id'] in form_ctx.objectIds())
            sub_form_item = form_ctx[form_field['id']]
            # make sure all the standard callables are set
            for k,v in form_field.items():
                if k == 'subfields':
                    self._verifyProfileForm(sub_form_item, v)
                else:
                    self.assertEqual(v, self._extractFieldValue(sub_form_item[k]),
                        "Expected '%s' for field %s, Got '%s'" % (v, k, sub_form_item[k]))
    

class TestFormExport(ExportImportTester):
    """Export Form Test Suite"""
    
    file_tmpl = 'structure/%s'
    title_output_tmpl = 'title: %s'
    
    def _getExporter(self):
        from Products.CMFCore.exportimport.content import exportSiteStructure
        return exportSiteStructure
    
    def test_stock_form_contextual_export(self):
        """Upon adding a form folder, we're given a fully functional
           example form that provides several fields, a mailer, and 
           a thanks page.  Let's make sure a contextual export includes what
           we're expecting.
        """
        self._makeForm()
        context = DummyExportContext(self.ff1)
        exporter = self._getExporter()
        exporter(context)
        
        # shove everything into a dictionary for easy inspection
        form_export_data = {}
        for filename, text, content_type in context._wrote:
            form_export_data[filename] = text
        
        # make sure our field and adapters are objects
        for id, object in self.ff1.objectItems():
            self.failUnless(form_export_data.has_key(self.file_tmpl % id), 
                    "No export representation of %s" % id)
            self.failUnless(self.title_output_tmpl % object.Title() in \
                    form_export_data[self.file_tmpl % id])
        
        # we should have .properties, .objects, and per subject
        self.assertEqual(len(context._wrote), 2 + len(self.ff1.objectIds()))
    
    def test_form_properties_contextual_export(self):
        """In order to accurately export the schema values for our 
           FormFolder, we forego the otherwise adequate 
           StructureFolderWalkingAdapter's ConfigParser format export 
           of .properties for favor of an RFC 822  (email) style string 
           export.  Confirm that is so here. 
        """
        self._makeForm()
        self.ff1.setSubmitLabel("Hit Me")
        context = DummyExportContext(self.ff1)
        exporter = self._getExporter()
        exporter(context)
        
        # shove everything into a dictionary for easy inspection
        form_export_data = {}
        for filename, text, content_type in context._wrote:
            form_export_data[filename] = text
        
        ff1_props = form_export_data['structure/.properties']
        
        lab_pat = re.compile(r'submitLabel.*?Hit Me')
        self.failUnless(lab_pat.search(ff1_props))
    
    def test_fieldset_properties_contextual_export(self):
        """In order to accurately export the schema values for our 
           FieldsetFolder, we forego the otherwise adequate 
           StructureFolderWalkingAdapter's ConfigParser format export 
           of .properties for favor of an RFC 822 (email) style string 
           export.  Confirm that is so here. 
        """
        self._makeForm()
        self.ff1.invokeFactory('FieldsetFolder','fsf1')
        self.ff1.fsf1.setTitle('FormFolder1 FieldsetFolder1')
        self.ff1.fsf1.setUseLegend(False) # set a non-default value
        context = DummyExportContext(self.ff1)
        exporter = self._getExporter()
        exporter(context)
        
        # shove everything into a dictionary for easy inspection
        form_export_data = {}
        for filename, text, content_type in context._wrote:
            form_export_data[filename] = text
        
        fsf1_props = form_export_data['structure/fsf1/.properties']
        
        self.failUnless('FormFolder1 FieldsetFolder1' in fsf1_props)
        leg_pat = re.compile(r'useLegend.*?False')
        self.failUnless(leg_pat.search(fsf1_props))
    
    def test_stock_form_view_export(self):
        """We provide a browser view that can be used to 
           export a given form as the root context as well.  
           XXX - Andrew B remember this is a rather tempoary representation
                 of what's returned.
        """
        toc_list = ['structure/.objects', 'structure/.properties', 'structure/mailer', 
                    'structure/replyto', 'structure/topic', 'structure/comments', 
                    'structure/thank-you']
        self._makeForm()
        form_folder_export = getMultiAdapter((self.folder.ff1, self.app.REQUEST), 
                                              name='export-form-folder')
        fileish = StringIO( form_folder_export() )
        self._verifyTarballContents( fileish, toc_list)
    

class TestFormImport(ExportImportTester):
    """Import Form Test Suite"""
    def _getImporter(self):
        from Products.CMFCore.exportimport.content import importSiteStructure
        return importSiteStructure
        
    def test_form_values_from_gs_import(self):
        """We provide a custom IFilesystemImporter so that
           all the schema fields from a configured FormFolder
           land in the imported form.
        """
        self.failUnless('test_form_1_form-folder' in self.folder.objectIds())
        self._verifyProfileFormSettings(self.folder['test_form_1_form-folder'])
    
    def test_profile_from_gs_import(self):
        """We create a profile (see: profiles/testing/structure) 
           which adds a FormFolder called 'test_form_1_' in via our 
           import handler. The form uses the standard ids proceeded by:
           test_form_1_.  We test for the existence of and accurate
           configuration of these subfields below.
        """        
        # did our gs form land into the test user's folder
        self.failUnless('test_form_1_form-folder' in self.folder.objectIds())
        self._verifyProfileForm(self.folder['test_form_1_form-folder'])
    
    def test_formlib_form_import(self):
        """Interacting with our formlib form we should be able
           to successfully upload an exported PFG form and have it
           correctly configured.
        """
        self._makeForm()
        
        # setup a reasonable request
        request = self.app.REQUEST
        request.form={
            'form.upload':self._prepareFormTarball(),
            'form.actions.import':'import'}
        request.RESPONSE = request.response
        
        # get the form object
        import_form = getMultiAdapter((self.ff1, request), name='import-form-folder')
        
        # call update (aka submit) on the form, see TestRequest above
        import_form.update()
        self._verifyProfileForm(self.ff1)
        self._verifyFormStockFields(self.ff1, purge=False)
    
    def test_formlib_form_with_purge_import(self):
        self._makeForm()
        
        # submit the form requesting purge of contained fields
        request = self.app.REQUEST
        request.form={
            'form.purge':u'on',
            'form.upload':self._prepareFormTarball(),
            'form.actions.import':'import'}
        request.RESPONSE = self.app.REQUEST.response
        
        # get the form object
        import_form = getMultiAdapter((self.ff1, request), name='import-form-folder')
        
        # call update (aka submit) on the form, see TestRequest above
        import_form.update()
        self._verifyProfileForm(self.ff1)
        self._verifyFormStockFields(self.ff1, purge=True)
    

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestFormExport))
    suite.addTest(makeSuite(TestFormImport))
    return suite



__author__ = """Titus Anderson <titus.anderson@louisville.edu>"""
__docformat__ = 'plaintext'

# Copyright (c) 2007 by Copyright (c) 2007 Titus Anderson
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

import cgi

from Products.Archetypes.public import *
from Products.Archetypes.utils import shasattr

from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.configuration import zconf

from Products.CMFCore.permissions import View, ModifyPortalContent

from Products.CMFPlone.utils import safe_hasattr

from AccessControl import ClassSecurityInfo

from Products.PloneFormGen.content import fieldsBase

from Products.PloneFormGen.content.likertField import LikertField
from Products.PloneFormGen.config import PROJECTNAME
from Products.PloneFormGen import PloneFormGenMessageFactory as _

from types import StringTypes

default_questions =  ('Question Number One','Question Number Two')
default_answers = (
    'Strongly disagree',
    'Disagree',
    'Neither agree nor disagree',
    'Agree',
    'Strongly agree'
    )

class FGLikertField(fieldsBase.BaseFormField):
    """ A Likert form entry """

    security = ClassSecurityInfo()

    schema = fieldsBase.BaseFieldSchema.copy() + Schema((
        LinesField('likertQuestions',
            searchable=0,
            required=1,
            default=default_questions,
            widget=LinesWidget(
                label='Questions',
                description = """List of questions; these will be the rows of the table.""",
                i18n_domain = "ploneformgen",
                label_msgid = "label_fglikert_questions",
                description_msgid = "help_fglikert_questions",
            ),
        ),
        LinesField('likertAnswers',
            searchable=0,
            required=1,
            default=default_answers,
            widget=LinesWidget(
                label='Answers',
                description = """List of possible answers for each of the questions;
                    these will be the columns of the table.""",
                i18n_domain = "ploneformgen",
                label_msgid = "label_fglikert_answers",
                description_msgid = "help_fglikert_answers",
            ),
        ),
    ))

    # 'hidden' isn't really useful for this field.
    del schema['hidden']
    # 'serverSide' is not really useful for this field.
    del schema['serverSide']

    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)

    portal_type = meta_type = "FormLikertField"
    archetype_name = "Rating-Scale Field"
    typeDescription = "A rating-scale implemented as rows of radio buttons."
    content_icon = 'LikertField.gif'

    def __init__(self, oid, **kwargs):
        """ Initialize Class """

        fieldsBase.BaseFormField.__init__(self, oid, **kwargs)

        self.fgField = LikertField('fg_likert_field',
            searchable = 0,
            required = 0,
            write_permission = View,
            questionSet = default_questions,
            answerSet = default_answers,
        )

    security.declareProtected(ModifyPortalContent, 'setLikertAnswers')
    def setLikertAnswers(self, value, **kwargs):
        if value in StringTypes:
            self.fgField.answerSet = tuple([a.strip() for a in value.split(',')])
        else:
            self.fgField.answerSet = tuple(value)

        self.likertAnswers = value

    security.declareProtected(ModifyPortalContent, 'setLikertQuestions')
    def setLikertQuestions(self, value, **kwargs):
        if value in StringTypes:
            self.fgField.questionSet = tuple([q.strip() for q in value.split(',')])
        else:
            self.fgField.questionSet = tuple(value)

        self.likertQuestions = value

    def htmlValue(self, REQUEST):
        """ return from REQUEST, this field's value, rendered as XHTML.
            In this case, a definition list.
        """

        value = REQUEST.form.get(self.__name__, 'No Input')
        if not (safe_hasattr(value, 'get') and 
                safe_hasattr(value, 'len') and 
                len(value)):
            return fieldsBase.BaseFormField.htmlValue(self, REQUEST)
        
        res = "<dl>\n"
        for i in range(len(value)):
            label = self.fgField.questionSet[i]
            response = value.get(str(i+1), '')
            res = "%s<dt>%s</dt><dd>%s</dd>\n" % (res, cgi.escape(label), cgi.escape(response))
        return "%s</dl>\n" % res


registerType(FGLikertField, PROJECTNAME)
